//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
using System;
using System.Diagnostics;
using System.Collections;
using System.Text;
using System.Web.Mail;

namespace TraceSwitchTest
{
	/// <summary>
	/// Summary description for TraceSwitchMain.
	/// </summary>
	// Based on the configuration output goes to
	// trace.log
	// event log
	// debug output stream
	// 
	class TraceSwitchMain
	{
		[Conditional("DEBUG")]
		static void WriteDebugOutput()
		{
			Console.WriteLine("This is debugging information");
		}

		public class CustomTraceListener : TraceListener
		{
			private Queue messageList;
			private StringBuilder message;
			private string mailTo;

			public CustomTraceListener(string to)
			{
				messageList = Queue.Synchronized(new Queue());
				mailTo = to;
			}
			public override void Close()
			{
				// The listener is closed
				Flush();

				MailMessage mailMessage = new MailMessage();
				mailMessage.To = mailTo;
				mailMessage.Subject = "Trace listing";
				mailMessage.BodyFormat = MailFormat.Text;

				while ( messageList.Count > 0 )
					mailMessage.Body += messageList.Dequeue() + "\r\n";

				mailMessage.Body += "\r\n";
				SmtpMail.Send(mailMessage);
			}
			public override void Flush()
			{
				// The listener is flushed
				if(message != null)
				{
					messageList.Enqueue(message);
					message = null;
				}
			}
			public override void Write(object o)
			{
				// Write an appendable message
				if(message == null)
					message = new StringBuilder();
				message.Append(o.ToString());
			}
			public override void Write(string s)
			{
				// Write an appendable message
				if(message == null)
					message = new StringBuilder();
				message.Append(s);
			}
			public override void Write(object o, string category)
			{
				// Write an appendable message
				if(message == null)
					message = new StringBuilder();
				message.Append(category + ": " + o.ToString());
			}
			public override void Write(string s, string category)
			{
				// Write an appendable message
				if(message == null)
					message = new StringBuilder();
				message.Append(category + ": " + s);
			}
			public override void WriteLine(object o)
			{
				Flush();
				// Write a message line
				messageList.Enqueue(o.ToString());
			}
			public override void WriteLine(string s)
			{
				Flush();
				// Write a message line
				messageList.Enqueue(s);
			}
			public override void WriteLine(object o, string s)
			{
				Flush();
				// Write a message line
				messageList.Enqueue(o.ToString() + s);
			}
			public override void WriteLine(string s, string category)
			{
				Flush();
				// Write a message line
				messageList.Enqueue(category + ": " + s);
			}
		}

		// The possible values for the new switch.
		public enum CustomTracingSwitchLevel 
		{
			Off = 0,
			ShowStopper = 1,
			Critical = 2,
			Severe = 3,
			Annoying = 4,
			Success = 5
		}

		public class CustomTracingSwitch : Switch 
		{
			public CustomTracingSwitch(string displayName, string description) : base(displayName, description)
			{
			}
 			public bool ShowStopper
			{
				get
				{
					return SwitchSetting >= (int)CustomTracingSwitchLevel.ShowStopper;
				}
			}
			public bool Critical
			{
				get
				{
					return SwitchSetting >= (int)CustomTracingSwitchLevel.Critical;
				}
			}
			public bool Severe
			{
				get
				{
					return SwitchSetting >= (int)CustomTracingSwitchLevel.Severe;
				}
			}
			public bool Annoying
			{
				get
				{
					return SwitchSetting >= (int)CustomTracingSwitchLevel.Annoying;
				}
			}
			public bool Success
			{
				get
				{
					return SwitchSetting >= (int)CustomTracingSwitchLevel.Success;
				}
			}
		}

 	    static void Main(string[] args)
		{
			CustomTraceListener ctl = new CustomTraceListener("first.last@mycompany.com");
			Trace.Listeners.Add(ctl);

			TraceSwitch ts1 = new TraceSwitch("MySwitch", "My debugging switch");
			TraceSwitch ts2 = new TraceSwitch("YourSwitch", "Your debugging switch");

			Trace.WriteLineIf(ts1.Level == TraceLevel.Info, "My output");
			Trace.WriteLineIf(ts1.TraceError, "My error output");
			Trace.WriteLineIf(ts1.TraceWarning, "My warning output");
			Trace.WriteLineIf(ts1.TraceInfo, "My info output");

			Trace.WriteLineIf(ts2.Level == TraceLevel.Error, "Your output");
			Trace.WriteLineIf(ts2.TraceError, "Your error output", "Category");

			CustomTracingSwitch cs = new CustomTracingSwitch("CustomSwitch", "Custom debugging switch");
			Trace.WriteLineIf(cs.Annoying, "Custom error output", "Custom category");

			WriteDebugOutput();
			// Trace.Flush();
			Trace.Close();
		}
	}
}
